######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    lookup_prep char(1), # Have the lookups been prepared?
    insert_prep char(1), # Has the insert statement been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=row, 2=field
    tab_pressed smallint # (boolean) was the tab key pressed?


######################################################################
function lld_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    define
        goto_top  smallint,  # (boolean) Go to the top of the array?
        in_insert smallint   # (boolean) true if we're in 'insert row'

    # Trap fatal errors
    whenever error call error_handler

    # Set some initial values
    let goto_top = true

    # This loop provides the ability to goto the top of the array.
    # Logic contained within the loop is not indented for readability.
    while goto_top

    let goto_top = false
    let in_insert = false
    let exit_level = 0
    let tab_pressed = false
    call set_count(rec1_cnt)
    input array p_gjourd without defaults from s_gjourd.*

      # The following section is for array type inputs
      before row
        if in_insert
        then
            # f1/f2 scenario - fix parallel array if pre-informix 4.0
            if not is_4_0() and rec1_cnt > 0
            then
                call lld_a_delete()
            end if
            let in_insert = false  # Reset in_insert flag
        end if
        let scr_fld = ""           # Reset the current field
        let p_cur = arr_curr()     # Set the current array line
        let s_cur = scr_line()     # Set the current screen line
        let rec1_cnt = arr_count() # Set the number of array elements
        call lld_b_row()
        goto next_field
      after row
        call lld_showline()
        call lld_a_row()
        goto next_field
      before insert
        let in_insert = true
        let rec1_cnt = arr_count() # Set the number of array elements
        call lld_b_insert()
        goto next_field
      after insert
        let in_insert = false
        call lld_a_insert()
        goto next_field
      before delete
        call lld_b_delete()
        goto next_field
      after delete
        call lld_a_delete()
        goto next_field

      # All entry fields must have before and after field processing
      before field acct_no call lld_b_field("stgjourd.acct_no")
        goto next_field
      before field department call lld_b_field("stgjourd.department")
        goto next_field
      before field tmpamt call lld_b_field("formonly.tmpamt")
        goto next_field
      before field amount call lld_b_field("stgjourd.amount")
        goto next_field
      before field debit_credit call lld_b_field("formonly.debit_credit")
        goto next_field

      after field acct_no call lld_a_field()
        goto next_field
      after field department call lld_a_field()
        goto next_field
      after field tmpamt call lld_a_field()
        goto next_field
      after field amount call lld_a_field()
        goto next_field
      after field debit_credit call lld_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 2 then call lld_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after row' logic if necessary
        if exit_level >= 1 then call lld_a_row() end if
        if nxt_fld is not null then goto next_field end if

        # Don't run 'after input' logic if looping to top
        if goto_top then exit input end if

        # Run the 'after input' logic before exiting input
        call lld_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_key("lld_input")  # Map the key to the event
        call lld_event()

      # Programmatic next field logic
      label next_field:
        case
          when nxt_fld is null  # No need to go through 'case'
          when nxt_fld = "acct_no" next field acct_no
          when nxt_fld = "department" next field department
          when nxt_fld = "tmpamt" next field tmpamt
          when nxt_fld = "amount" next field amount
          when nxt_fld = "debit_credit" next field debit_credit
          when nxt_fld = "exit input" goto end_input
          when nxt_fld = "goto top"
            call mld_arr_count()
            let goto_top = true
            goto end_input
        end case

    end input

    # This is the end of the 'goto top' loop.
    end while

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# lld_input()

######################################################################
function lld_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field and row level exit
    let exit_level = 2

    # Specify fields only if work needs to be done.
    case
      when scr_fld = "acct_no"
    let scratch = "zoom"
      when scr_fld = "department"
    let scratch = "zoom"
    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# lld_b_field()

######################################################################
function lld_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # Require row level exit
    let exit_level = 1

    # After field logic
    case
      when scr_fld = "acct_no"
        if auto_zoom("chrtz","stxchrtr","")
          then let p_gjourd[p_cur].acct_no = scratch end if

      when scr_fld = "department"
        call fgda_department()

    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # This (among other things) sets data_changed
    call lib_after()

    # After data_changed logic
    if data_changed
    then
        case
          when scr_fld = "acct_no"
            # Perform lookups
            if lld_lookup("acct_look",true) = false and
              length(this_data) != 0
            then
                let nxt_fld = "acct_no"
                return
            end if
            call fgda_acct_no()
          when scr_fld = "department"
            # Perform lookups
            if lld_lookup("dept_look",true) = false and
              length(this_data) != 0
            then
                let nxt_fld = "department"
                return
            end if
          when scr_fld = "tmpamt"
            call fgda_tmpamt()
          when scr_fld = "amount"
            call fgda_amount()
          when scr_fld = "debit_credit"
            call fgda_credit_debit()
        end case
    end if

end function
# lld_a_field()

######################################################################
function lld_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#

    if not tab_pressed
    then
        call lld_chk_p_array()
        if rec1_cnt = 0
        then
            call fg_err(2) # Document must have detail line(s).
            let tab_pressed = true
            return
        end if
    end if
    call lld_recalc()
    if p_gstder.diff <> 0
    then
        open window diffwin at 19, 8 with 1 rows, 66 columns
        attribute (border, white)
        let scratch = " Credit= ",p_gstder.crtotal
        let scratch = scratch clipped," Debit= ",p_gstder.dbtotal
        let scratch = scratch clipped," Diff= ",p_gstder.diff
        display scratch at 1,2 attribute(white)
        call fg_err(1) # Credits and Debits are not balance
        close window diffwin
        let nxt_fld = "goto top"
    end if
end function
# lld_a_input()

######################################################################
function lld_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "zoom" and infield(acct_no)
        if zoom("chrtz","")
        then
            let p_gjourd[p_cur].acct_no = scratch
            let nxt_fld = "acct_no"
        end if
      when scr_funct = "zoom" and infield(department)
        let scratch =
          " stxinfor.src_type = \"D\""
        if zoom("deptz","(see scratch)")
        then
            let p_gjourd[p_cur].department = scratch
            let nxt_fld = "department"
        end if
      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# lld_event()

######################################################################
function lld_b_row()
######################################################################
# This function is called before you enter a new row.
#

end function
# lld_b_row()

######################################################################
function lld_a_row()
######################################################################
# This function is called whenever you leave a row.
#
    # No more exit levels required
    let exit_level = 0

end function
# lld_a_row()

######################################################################
function lld_b_insert()
######################################################################
# This function is called before a new row is added to the array.
#
    define
        n smallint   # Generic counter

    # Expand (shift) the parallel array
    for n = (rec1_cnt - 1) to p_cur step -1
        let q_gjourd[n+1].* = q_gjourd[n].*
    end for

    # Blank out the current array element
    initialize q_gjourd[p_cur].* to null

    let scr_fld = null
end function
# lld_b_insert()

######################################################################
function lld_a_insert()
######################################################################
# This function is called after a new row has been added to the array.
#

end function
# lld_a_insert()

######################################################################
function lld_b_delete()
######################################################################
# This function is called when the user presses [F2] (delete a row),
# and before the actual array elements have been shifted.
#

end function
# lld_b_delete()

######################################################################
function lld_a_delete()
######################################################################
# This function is called when the user presses [F2] (delete a row),
# and after the actual array elements have been shifted.
# (rec1_cnt contains the number of elements before the delete)
#
    define
        n smallint   # Generic counter

    # Compress (shift) the parallel array
    for n = (p_cur + 1) to rec1_cnt
        let q_gjourd[n-1].* = q_gjourd[n].*
    end for

    # Blank out the last (duplicate) data element
    initialize p_gjourd[rec1_cnt].* to null
    initialize q_gjourd[rec1_cnt].* to null

    # Perform array math
    call llh_math()
    call llh_display()

    call lld_recalc()
end function
# lld_a_delete()

######################################################################
function lld_setdata()
######################################################################
# This function is called to set the this_data variable
#
    case
      when scr_fld = "acct_no"
        call set_data(p_gjourd[p_cur].acct_no)
      when scr_fld = "department"
        call set_data(p_gjourd[p_cur].department)
      when scr_fld = "tmpamt"
        call set_data(p_gjourd[p_cur].tmpamt)
      when scr_fld = "amount"
        call set_data(p_gjourd[p_cur].amount)
      when scr_fld = "debit_credit"
        call set_data(p_gjourd[p_cur].debit_credit)
    end case
end function
# lld_setdata()

######################################################################
function lld_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "acct_no"
        display p_gjourd[p_cur].acct_no
          to s_gjourd[s_cur].acct_no attribute(reverse)
      when scr_fld = "department"
        display p_gjourd[p_cur].department
          to s_gjourd[s_cur].department attribute(reverse)
      when scr_fld = "tmpamt"
        display p_gjourd[p_cur].tmpamt
          to s_gjourd[s_cur].tmpamt attribute(reverse)
      when scr_fld = "amount"
        display p_gjourd[p_cur].amount
          to s_gjourd[s_cur].amount attribute(reverse)
      when scr_fld = "debit_credit"
        display p_gjourd[p_cur].debit_credit
          to s_gjourd[s_cur].debit_credit attribute(reverse)
    end case
end function
# lld_high()

######################################################################
function lld_showline()
######################################################################
# This function displays the current p_* variables onto the screen line
#
    display p_gjourd[p_cur].* to s_gjourd[s_cur].*
end function
# lld_showline()

######################################################################
function lld_display()
######################################################################
# This function displays the first screenful of data
#
    for p_cur = 1 to scr1_max
        display p_gjourd[p_cur].* to s_gjourd[p_cur].*  attribute(red)
    end for
end function
# lld_display()

######################################################################
function lld_p_prep(n)
######################################################################
# This function creates the p_[]* record from the m_* record
#
    define
        n smallint      # Array element to process

    let p_gjourd[n].acct_no = m_gjourd.acct_no
    let p_gjourd[n].department = m_gjourd.department
    let p_gjourd[n].amount = m_gjourd.amount

    # Perform lookups
    let p_cur = n
    if lld_lookup("acct_look",false) then end if
    if lld_lookup("dept_look",false) then end if

    if m_gjourd.debit_credit = "C"
    then
        let p_gjourd[n].debit_credit = "CR"
    else
        let p_gjourd[n].debit_credit = "DB"
    end if
    if p_cur_scr = 1
    then
        if (q_gjourd[n].incr_with_crdt = "Y"
            and p_gjourd[n].debit_credit = "CR")
          or (q_gjourd[n].incr_with_crdt = "N"
              and p_gjourd[n].debit_credit = "DB")
        then
            let p_gjourd[n].tmpamt = p_gjourd[n].amount
        else
            let p_gjourd[n].tmpamt = p_gjourd[n].amount * -1
        end if
    end if

end function
# lld_p_prep()

######################################################################
function lld_m_prep(n)
######################################################################
# This function creates the m_* record from the specified element
# of the p_.* array, and from elements of the header that tie the
# detail lines to the header
#
    define
        n smallint      # Array element to process

    # Items from the header...
    let m_gjourd.doc_no = m_gstder.doc_no

    # Items from the p_ record array...
    let m_gjourd.acct_no = p_gjourd[n].acct_no
    let m_gjourd.department = p_gjourd[n].department
    let m_gjourd.amount = p_gjourd[n].amount

    # Auto line numbering
    let m_gjourd.line_no = n

    let m_gjourd.orig_journal = "SJ"
    let m_gjourd.debit_credit = p_gjourd[n].debit_credit[1,1]

end function
# lld_m_prep()

######################################################################
function lld_delete()
######################################################################
# This function deletes the rows in the detail table
# that match the where clause
#
    delete from stgjourd
      where stgjourd.doc_no = m_gstder.doc_no
        and stgjourd.orig_journal = "SJ"
end function
# lld_delete()

######################################################################
function lld_add()
######################################################################
# This function inserts data into the detail table.
#
    # Declare an insert cursor for maximum efficiency
    if insert_prep is null
    then
        let insert_prep = "Y"
        declare d_insert cursor for
          insert into stgjourd values(m_gjourd.*)
    end if

    # Insert all array elements
    open d_insert
    for p_cur = 1 to rec1_cnt
        call lld_m_prep(p_cur)
        put d_insert
    end for
    close d_insert

end function
# lld_add()

######################################################################
function lld_lookup(tbl_name,mustfind)
# returning false if notfound
######################################################################
# This is the lookup function.  It is passed a table to lookup, and
# based on that, it looks up the key and fills any variables for
# that table.  The lookup function will produce the error if it is
# called with mustfind=true, and a notfound situation exists.
#
    define
        tbl_name char(18),  # Lookup table name
        stat integer,       # Value of sqlca.sqlcode after lookup
        mustfind smallint   # (boolean) produce an error if notfound?

    # Prepare cursors upon first call to this function
    if lookup_prep is null
    then
        let lookup_prep = "Y"
        let scratch = "select acct_desc, incr_with_crdt ",
          "from stxchrtr where ",
          " stxchrtr.acct_no = ?"
        prepare str_acct_look from scratch
        declare cur_acct_look cursor for str_acct_look
        let scratch = "select rowid ",
          "from stxinfor where ",
          " stxinfor.src_key = ? and",
          " stxinfor.src_type = \"D\""
        prepare str_dept_look from scratch
        declare cur_dept_look cursor for str_dept_look
    end if

    # Perform the lookup
    case
      when tbl_name = "acct_look"
        open cur_acct_look using p_gjourd[p_cur].acct_no
        fetch cur_acct_look into p_gjourd[p_cur].acct_desc,
          q_gjourd[p_cur].incr_with_crdt
        let stat = sqlca.sqlcode
        close cur_acct_look
      when tbl_name = "dept_look"
        open cur_dept_look using p_gjourd[p_cur].department
        fetch cur_dept_look into scratch
        let stat = sqlca.sqlcode
        close cur_dept_look
    end case

    # No mustfind if this_data is empty
    if length(this_data) = 0 then let mustfind = false end if

    # Send notfound message to user (if requested)
    let sqlca.sqlcode = stat
    if mustfind and stat = notfound
      then call scr_error("lookup=notfound",tbl_name)
    end if
    let sqlca.sqlcode = 0

    # Return false if lookup failed and empty into fields
    if stat = notfound
    then
        case
          when tbl_name = "acct_look"
            initialize p_gjourd[p_cur].acct_desc,
              q_gjourd[p_cur].incr_with_crdt to null
          when tbl_name = "dept_look"
            initialize scratch to null
        end case
        return false
    else
        return true
    end if
end function
# lld_lookup()


######################################################################
function fgda_acct_no()
# after field logic for the detail field acct_no
######################################################################
#

    if p_gjourd[p_cur].acct_no is null
    then
        initialize p_gjourd[p_cur].* to null
        initialize q_gjourd[p_cur].* to null
    else
        let p_gjourd[p_cur].amount = null
        let p_gjourd[p_cur].tmpamt = null
    end if
    call lld_recalc()
    if q_gjourd[p_cur].incr_with_crdt is not null
    then
        if q_gjourd[p_cur].incr_with_crdt = "Y"
          then let p_gjourd[p_cur].debit_credit = "CR"
          else let p_gjourd[p_cur].debit_credit = "DB" end if
    end if

end function
# fgda_acct_no

######################################################################
function fgda_department()
# after field logic for the detail field department
######################################################################
#

    if p_gjourd[p_cur].acct_no is not null and
       p_gjourd[p_cur].department is null
    then
        let p_gjourd[p_cur].department = "000"
        display p_gjourd[p_cur].department
             to s_gjourd[s_cur].department
    end if

end function
# fgda_department

######################################################################
function fgda_amount()
# after field logic for the detail field amount
######################################################################
#

    let p_gjourd[p_cur].amount = round("a", p_gjourd[p_cur].amount)
    let p_gjourd[p_cur].amount = absolute(p_gjourd[p_cur].amount)

    if p_gjourd[p_cur].acct_no is not null and
       p_gjourd[p_cur].department is not null
    then
        call lld_recalc()
    else
        initialize p_gjourd[p_cur].* to null
        display p_gjourd[p_cur].* to s_gjourd[s_cur].*
    end if

end function
# fgda_amount

######################################################################
function fgda_credit_debit()
# after field logic for the detail field credit_debit
######################################################################
#

    if p_gjourd[p_cur].acct_no is not null
    then
       if (p_gjourd[p_cur].debit_credit = "DB" or
           p_gjourd[p_cur].debit_credit = "CR")
       then
           call lld_recalc()
       else
           call fg_err(4)  # Invalid entry
           let nxt_fld = "debit_credit"
       end if
    end if

end function
# fgda_credit_debit

######################################################################
function fgda_tmpamt()
# after field logic for the detail field tmpamt
######################################################################
#

    let p_gjourd[p_cur].tmpamt = round("a",
        p_gjourd[p_cur].tmpamt)

    if p_gjourd[p_cur].acct_no is not null and
       p_gjourd[p_cur].department is not null
    then
        call flip()
        call lld_recalc()
    else
        initialize p_gjourd[p_cur].* to null
        display p_gjourd[p_cur].* to s_gjourd[s_cur].*
    end if

end function
# fgda_tmpamt

############################################################
function lld_recalc()
#     returns the difference between the debits and credits
############################################################
# Recalculate totals of debits and credits found on bottom
# of screen.
#

    define
        counter             smallint,
        tot_credit          like stgjourd.amount,
        tot_debit           like stgjourd.amount,
        tot_diff            like stgjourd.amount

    let tot_credit = 0
    let tot_debit = 0
    let tot_diff = 0

    for counter = 1 to rec1_cnt
      if p_gjourd[counter].amount is not null then
          if p_gjourd[counter].debit_credit = "CR" then
              let tot_credit=p_gjourd[counter].amount + tot_credit
          else
              let tot_debit=p_gjourd[counter].amount + tot_debit
          end if
      end if
    end for

    let tot_diff = tot_credit - tot_debit
    if tot_diff < 0
    then
        let tot_diff = tot_diff * -1
    end if

    let p_gstder.crtotal = tot_credit
    let p_gstder.dbtotal = tot_debit
    let p_gstder.diff = tot_diff

    call llh_display()

end function
# lld_recalc()

#######################################################################
function lld_chk_p_array()
#######################################################################
#   - this function removes lines that are null or zero
#
    define
        num_in smallint,     # outer loop counter
        n smallint,     # inner loop counter
        num_out smallint      # inner loop counter

    let num_in = 0
    let num_out = 0
    for n = 1 to rec1_cnt
      if p_gjourd[n].amount is null or p_gjourd[n].amount = 0
      then
          let num_out = num_out + 1
          continue for
      end if
      let num_in = num_in + 1
      let p_gjourd[num_in].* = p_gjourd[n].*
      let q_gjourd[num_in].* = q_gjourd[n].*
    end for
    initialize p_gjourd[num_in + 1].* to null
    initialize q_gjourd[num_in + 1].* to null
    for n = (num_in + 2) to rec1_cnt
      let p_gjourd[n].* = p_gjourd[num_in + 1].*
      let q_gjourd[n].* = q_gjourd[num_in + 1].*
    end for
    let rec1_cnt = num_in
    call lld_display()

end function
# lld_chk_p_array()
