######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

######################################################################
function mlh_init()
# returning the initial find group filter string.
######################################################################
# This function serves two purposes.
# It is called only once upon program invocation, so you may use
# it as a place to run initialization routines.
# It requires a return value of the default select group.
# examples of return values:
#   return "1=1" (select all rows)
#   return "1=0" (select no rows)
#   return "customer_num is not null" (select non-null customer_num's)
#   return "posted = \"N\"" (select only unposted documents)
#
    # Trap fatal errors
    whenever error call error_handler

    return "1=0"
end function
# mlh_init()

######################################################################
function mlh_clear()
######################################################################
#
    initialize p_gstder.* to null
    initialize q_gstder.* to null
    initialize m_gstder.* to null
end function
# mlh_clear

######################################################################
function mld_clear()
######################################################################
#
    define
        n       smallint            # working number

    initialize m_gjourd.* to null    # clear the m_record
    initialize p_gjourd[1].* to null # clear the first p_record
    initialize q_gjourd[1].* to null # clear the first p_record
    for n = 2 to rec1_cnt
        let p_gjourd[n].* = p_gjourd[1].* # clear subsequent p_records
        let q_gjourd[n].* = q_gjourd[1].* # clear subsequent q_records
    end for
    let rec1_cnt = 0                # reset the active element counter
end function
# mld_mclear()

######################################################################
function mlh_construct()
# returning criteria
######################################################################
#
    construct scratch
      on
        stgstder.post_type, stgstder.doc_desc, stgstder.eop_rev,
        stgstder.doc_src
      from
        s_gstder.post_type, s_gstder.doc_desc, s_gstder.eop_rev,
        s_gstder.doc_src
    return scratch[1,512]
end function
# mlh_construct

######################################################################
function mlh_cursor()
######################################################################
# This function defines the table, filter, and ordering portions of
# the select statement used to build the FourGen scroller.
#
    call put_vararg("stgstder")
    call put_vararg("")
    call put_vararg("doc_no")
end function
# mlh_cursor()

######################################################################
function mld_scroll()
######################################################################
# This function calls display array for the scrolling input.
#
    if rec1_cnt < 1 then return end if
    call set_count(rec1_cnt)
    display array p_gjourd to s_gjourd.* attribute(red)
      on key (control-m,control-i)
        exit display
    end display
end function
# mld_scroll

######################################################################
function mld_arr_count()
######################################################################
# This function sets rec1_cnt to the correct number of array elements.
#
    define
        n smallint
    for n = arr_count() to 1 step -1
        if  p_gjourd[n].acct_no is not null
         or p_gjourd[n].department is not null
         or p_gjourd[n].acct_desc is not null
         or p_gjourd[n].tmpamt is not null
         or p_gjourd[n].amount is not null
         or p_gjourd[n].debit_credit is not null
         or q_gjourd[n].row_id is not null
         or q_gjourd[n].incr_with_crdt is not null
        then
            let rec1_cnt = n
            return
        end if
    end for
    let rec1_cnt = 0
end function
# mld_arr_count()

######################################################################
function mlh_key()
# returning tablename, key_field [,key_field]... in (vararg) scratch
######################################################################
# This function returns (in vararg scratch) the name of the main table
# for this screen and the field name(s) and their data that build
# the unique key into this table.
# This is used to determine the unique key for additional fields,
# notes, and other items that may key off this document.  The key
# field(s) must not exceed 30 characters total.
# The first vararg call is the table name.  The following varargs are
# called in pairs.  The first of the pair is the column name, and
# the second in the pair is the column data.
#
    call put_vararg("stgstder")
    call put_vararg("doc_no")
    call put_vararg(m_gstder.doc_no)
end function
# mlh_key()
